@@ -1,5 +1,5 @@
 // Copyright (C) 2022 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -53,6 +53,7 @@ private slots:
 
     void sayMultiple_data();
     void sayMultiple();
+    void sayInvalid();
 
     void pauseAtUtterance_data();
     void pauseAtUtterance();
@@ -553,7 +554,7 @@ void tst_QTextToSpeech::pauseResume()
     QFETCH_GLOBAL(QString, engine);
     if (engine != "mock" && !hasDefaultAudioOutput())
         QSKIP("No audio device present");
-    if (engine == "macos" || engine == "speechd")
+    if (engine == "speechd")
         QSKIP("Native speech engine is faulty");
 
     const QString text = QStringLiteral("Hello. World.");
@@ -661,6 +662,33 @@ void tst_QTextToSpeech::sayWithRates()
     logger.dismiss();
 }
 
+void tst_QTextToSpeech::sayInvalid()
+{
+    QFETCH_GLOBAL(QString, engine);
+    if (engine != "mock" && !hasDefaultAudioOutput())
+        QSKIP("No audio device present");
+
+    QTextToSpeech tts(engine);
+    QTRY_COMPARE(tts.state(), QTextToSpeech::Ready);
+    selectWorkingVoice(&tts);
+    auto logger = qScopeGuard([&tts]{
+        qWarning() << "Failure with voice" << tts.voice();
+    });
+
+    int speakingCount = 0;
+    bool doneSpeaking = false;
+    connect(&tts, &QTextToSpeech::stateChanged, this, [&](QTextToSpeech::State state){
+        if (state == QTextToSpeech::Speaking)
+            ++speakingCount;
+        else if (state == QTextToSpeech::Ready)
+            doneSpeaking = true;
+    });
+
+    tts.say("©");
+    QTRY_VERIFY(doneSpeaking);
+    QTRY_COMPARE(speakingCount, 1);
+}
+
 void tst_QTextToSpeech::sayMultiple_data()
 {
     QTest::addColumn<QStringList>("textList");
@@ -695,7 +723,7 @@ void tst_QTextToSpeech::sayMultiple()
     QFETCH(const QStringList, textList);
     for (qsizetype i = 0; i < textList.count(); ++i) {
         const QString &text = textList.at(i);
-        tts.enqueue(text);
+        qDebug() << "Equeued utterance" << tts.enqueue(text);
         if (!i) // wait for the engine to start synthesizing
             QTRY_COMPARE_NE(tts.state(), QTextToSpeech::Ready);
     }
@@ -704,6 +732,7 @@ void tst_QTextToSpeech::sayMultiple()
     QCOMPARE(aboutToSynthesizeSpy.count(), textList.size());
     QCOMPARE(speakingCount, 1);
 
+    qDebug() << "Synthesized utterances" << aboutToSynthesizeSpy;
     logger.dismiss();
 }
 
@@ -726,7 +755,7 @@ void tst_QTextToSpeech::pauseAtUtterance()
 
     int atIndex = -1;
     bool paused = false;
-    connect(&tts, &QTextToSpeech::aboutToSynthesize, [&]{
+    connect(&tts, &QTextToSpeech::aboutToSynthesize, this, [&]{
         ++atIndex;
         if (atIndex == 1 && !paused) {
             tts.pause(QTextToSpeech::BoundaryHint::Utterance);
@@ -735,7 +764,7 @@ void tst_QTextToSpeech::pauseAtUtterance()
         }
     });
     QStringList wordsSpoken;
-    connect(&tts, &QTextToSpeech::sayingWord,
+    connect(&tts, &QTextToSpeech::sayingWord, this,
             [&](const QString &word, qsizetype id, qsizetype at, qsizetype length){
         const QString text = textList.at(atIndex).mid(at, length);
         QCOMPARE(id, atIndex);
@@ -792,14 +821,15 @@ void tst_QTextToSpeech::sayingWord()
 
     QElapsedTimer timer;
     QStringList words;
-    QList<qint64> times;
-    connect(&tts, &QTextToSpeech::sayingWord,
+
+    QList<std::chrono::nanoseconds> times;
+    connect(&tts, &QTextToSpeech::sayingWord, this,
         [&words, &times, &timer, text](const QString &word, qsizetype id, qsizetype start, qsizetype length) {
         const QString &slice = text.sliced(start, length);
         QCOMPARE(word, slice);
         QCOMPARE(id, 0);
         words << text.sliced(start, length);
-        times << timer.elapsed();
+        times << timer.durationElapsed();
     });
 
     timer.start();
@@ -810,7 +840,7 @@ void tst_QTextToSpeech::sayingWord()
     });
     QTRY_COMPARE(tts.state(), QTextToSpeech::Speaking);
     QTRY_COMPARE(tts.state(), QTextToSpeech::Ready);
-    qint64 totalTime = timer.elapsed();
+    auto totalTime = timer.durationElapsed();
 
     QCOMPARE(words, expectedWords);
 
@@ -821,6 +851,13 @@ void tst_QTextToSpeech::sayingWord()
     if (words.count() > 1)
         QCOMPARE_GE(times.last(), totalTime * 0.4);
 
+    std::list<std::chrono::nanoseconds> timeDifferences;
+    std::adjacent_difference(times.begin(), times.end(), std::back_inserter(timeDifferences));
+    timeDifferences.pop_front(); // first element is always 0
+
+    for (auto timeDiff : timeDifferences)
+        QCOMPARE_GT(timeDiff, std::chrono::milliseconds(5));
+
     debugHelper.dismiss();
 }
 
@@ -839,8 +876,10 @@ void tst_QTextToSpeech::sayingWordWithPause()
     QFETCH_GLOBAL(QString, engine);
     if (engine != "mock" && !hasDefaultAudioOutput())
         QSKIP("No audio device present");
-    if (engine == "macos")
-        QSKIP("macos engine's pause support is faulty");
+
+    if (engine == "flite")
+        QSKIP("QTBUG-137947 QTextToSpeech::pause(QTextToSpeech::BoundaryHint::Word) not implemented"
+              " for flite");
 
     QFETCH(QStringList, words);
     QFETCH(int, pauseAt);
@@ -856,7 +895,7 @@ void tst_QTextToSpeech::sayingWordWithPause()
     selectWorkingVoice(&tts);
 
     QStringList spokenWords;
-    connect(&tts, &QTextToSpeech::sayingWord,
+    connect(&tts, &QTextToSpeech::sayingWord, this,
             [&](const QString &word, qsizetype id, qsizetype start, qsizetype length) {
         QCOMPARE(word, text.sliced(start, length));
         QCOMPARE(id, 0);
@@ -920,7 +959,8 @@ void tst_QTextToSpeech::synthesize()
     bool running = false;
     bool finished = false;
     qint64 speechTime = 0;
-    connect(&tts, &QTextToSpeech::stateChanged, [&running, &finished, &speechTimer, &speechTime](QTextToSpeech::State state) {
+    connect(&tts, &QTextToSpeech::stateChanged, this,
+            [&running, &finished, &speechTimer, &speechTime](QTextToSpeech::State state) {
         if (state == QTextToSpeech::Synthesizing || state == QTextToSpeech::Speaking) {
             speechTimer.start();
             running = true;
@@ -963,8 +1003,8 @@ void tst_QTextToSpeech::synthesize()
         const qint32 bytesExpected = pcmFormat.bytesForDuration(speechTime * 1000);
 
         // We should have as much data as the format requires for the time it took
-        // to play the speech, +/- 10% as we can't measure the exact audio duration.
-        QCOMPARE_GE(pcmData.size(), double(bytesExpected) * 0.9);
+        // to play the speech, +/- 15% as we can't measure the exact audio duration.
+        QCOMPARE_GE(pcmData.size(), double(bytesExpected) * 0.85);
         if (engine == "flite") // flite is very unreliable
             QCOMPARE_LT(pcmData.size(), double(bytesExpected) * 1.5);
         else
